# generated by scripts/generate_messages.py

from twnet_parser.pretty_print import PrettyPrint
from typing import Literal

class MsgRequestCount(PrettyPrint):
    def __init__(
            self
    ) -> None:
        self.message_type: Literal['connless'] = 'connless'
        self.message_name: str = 'connless.request_count'
        self.message_id: list[int] = [255, 255, 255, 255, 99, 111, 117, 50]


    def __iter__(self):
        yield 'message_type', self.message_type
        yield 'message_name', self.message_name
        yield 'message_id', self.message_id

    # first byte of data
    # has to be the first byte of the message payload
    # NOT the chunk header and NOT the message id
    def unpack(self, data: bytes) -> bool:
        return True

    def pack(self) -> bytes:
        return b''