PLAYER_INPUT = 1
PROJECTILE = 2
LASER = 3
PICKUP = 4
FLAG = 5
GAME_DATA = 6
GAME_DATA_TEAM = 7
GAME_DATA_FLAG = 8
CHARACTER_CORE = 9
CHARACTER = 10
PLAYER_INFO = 11
SPECTATOR_INFO = 12
DE_CLIENT_INFO = 13
DE_GAME_INFO = 14
DE_TUNE_PARAMS = 15
COMMON = 16
EXPLOSION = 17
SPAWN = 18
HAMMERHIT = 19
DEATH = 20
SOUND_WORLD = 21
DAMAGE = 22
PLAYER_INFO_RACE = 23
GAME_DATA_RACE = 24
